import 'dart:convert';

import 'package:http/http.dart';

class ApiResponse<T> {
  static withException<T>(Exception error) {
    return ApiErrorResponse<T>(error?.toString() ?? 'Unknown error');
  }

  static withResponse<T>(Response response, T bodyResponse) {
    if (response.statusCode == 200) {
      final body = response.body;
      if (body == null || response.statusCode == 204) {
        return ApiEmptyResponse<T>();
      } else {
        return ApiSuccessResponse<T>(bodyResponse);
      }
    } else {
      var error;
      try {
        var j = json.decode(response.body);
        if (j['error'] != null) {
          error = j['error'];
        } else if (j['message'] != null) {
          error = j['message'];
        }
      } catch (e, s) {
        print(s);
      }
      return ApiErrorResponse<T>(error ?? response.body ?? 'Unknown error');
    }
  }
}

class ApiEmptyResponse<T> extends ApiResponse<T> {}

class ApiSuccessResponse<T> extends ApiResponse<T> {
  T body;

  ApiSuccessResponse(this.body);
}

class ApiErrorResponse<T> extends ApiResponse<T> {
  String errorMessage;

  ApiErrorResponse(this.errorMessage);
}
