import 'dart:async';

import 'package:bloc_event_gen/bloc_event_gen.dart';
import 'package:flutter/material.dart';

/// A base BLoC implementation.
abstract class Bloc {
  /// A Stream that notifies the UI with events for it to handle. This is used
  /// for actions like navigating to a different route, showing a dialog, and
  /// similar, which the BLoC can't handle due to lack of [BuildContext].
  ///
  /// Don't forget to cancel your subscription to this stream once you are done.
  /// Or just use [BlocUserState], which handles subscribing to this stream,
  /// provides a nice API for defining event handling, and cancels subscription
  /// for you.
  final _uiEventsController = StreamController<BlocEvent>.broadcast();

  Stream<BlocEvent> get uiEvents => _uiEventsController.stream;

  /// Called when the bloc is attached to a widget. Might be called more than
  /// once.
  void onAttach() {}

  void dispatchUiEvent(BlocEvent event) {
    _uiEventsController.sink.add(event);
  }

  void dispose() {
    _uiEventsController.close();
  }
}
