import 'package:arch_foundations/arch/bloc.dart';
import 'package:arch_foundations/arch/bloc_provider.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';

class TestBloc1 extends Mock implements Bloc {}

class TestBloc2 extends Mock implements Bloc {}

class TestBloc3 extends Mock implements Bloc {}

void main() {
  final bloc1 = TestBloc1();
  final bloc2 = TestBloc2();
  final bloc3 = TestBloc3();

  testWidgets('provides bloc of given type', (tester) async {
    await tester.pumpWidget(BlocProvider(
      blocs: [
        bloc1,
        bloc2,
      ],
      child: _TestWidget(),
    ));

    final _TestWidgetState state = tester.state(find.byType(_TestWidget));

    expect(state.obtainBloc<TestBloc2>(), isInstanceOf<TestBloc2>());
  });

  testWidgets('throws if asked for unregistered bloc type', (tester) async {
    await tester.pumpWidget(BlocProvider(
      blocs: [
        bloc1,
        bloc2,
      ],
      child: _TestWidget(),
    ));

    final _TestWidgetState state = tester.state(find.byType(_TestWidget));

    expect(
      () => state.obtainBloc<TestBloc3>(),
      throwsA(isInstanceOf<UnimplementedError>()),
    );
  });
}

class _TestWidget extends StatefulWidget {
  @override
  _TestWidgetState createState() => _TestWidgetState();
}

class _TestWidgetState extends State<_TestWidget> {
  @override
  Widget build(BuildContext context) {
    return Container();
  }

  Bloc obtainBloc<T extends Bloc>() => BlocProvider.of<T>(context);
}
