import 'dart:async';

import 'package:arch_foundations/arch/dependant_stream.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  test(
      'correctly emits from initialSource when dependency doesn\'t '
      'emit anything', () {
    final depStream = DependantStream<int, String>();
    final dependency = Stream<int>.fromIterable([]);
    final source1 = Stream.fromIterable(['1']);
    final source2 = Stream.fromIterable(['2']);

    expect(depStream.stream, emits('2'));

    depStream.dependOn(
      dependency: dependency,
      initialSource: source2,
      sourceBuilder: (dependencyValue) {
        return dependencyValue == 1 ? source1 : source2;
      },
    );
  });

  test('emits from the correct source', () async {
    final depStream = DependantStream<int, String>();
    final dependencyController = StreamController<int>();
    final dependency = dependencyController.stream;
    final source1 = Stream.fromIterable(['1']).asBroadcastStream();
    final source2 = Stream.fromIterable(['2']).asBroadcastStream();

    depStream.dependOn(
      dependency: dependency,
      initialSource: source2,
      sourceBuilder: (dependencyValue) {
        return dependencyValue == 1 ? source1 : source2;
      },
    );

    dependencyController.sink.add(1);

    // Wait for the dependency to emit and for depStream to update its source
    await Future.delayed(Duration(milliseconds: 10));

    expect(depStream.stream, emits('1'));

    dependencyController.close();
  });
}
