import 'package:arch_foundations/arch/network_bound_resource_client.dart';
import 'package:arch_foundations/resource.dart';
import 'package:flutter/src/material/snack_bar.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_test/flutter_test.dart';

class _TestClient with NetworkBoundResourceClient {
  @override
  void showSnackBar(String text, [SnackBarAction action]) {}
}

void main() {
  test('calls errorBuilder when snapshot has error', () {
    var called = false;
    final builder = _TestClient().buildForStream(
      contentBuilder: null,
      errorBuilder: (error) {
        called = true;
      },
      loadingBuilder: null,
      emptyBuilder: null,
    );

    builder(null, AsyncSnapshot.withError(ConnectionState.done, 'error'));

    expect(called, isTrue);
  });

  test(
      'calls promptLoading when resource is loading, has data, and '
      'promptLoading is not null', () {
    var called = false;
    final builder = _TestClient().buildForStream(
      contentBuilder: (data) {},
      errorBuilder: null,
      loadingBuilder: null,
      emptyBuilder: null,
      promptLoading: () {
        called = true;
      },
    );

    builder(
      null,
      AsyncSnapshot.withData(
        ConnectionState.done,
        Resource.loading('some data'),
      ),
    );

    expect(called, isTrue);
  });

  test('calls loadingBuilder when resource is loading and has no data', () {
    var called = false;
    final builder = _TestClient().buildForStream(
      contentBuilder: null,
      errorBuilder: null,
      loadingBuilder: () {
        called = true;
      },
      emptyBuilder: null,
      promptLoading: () {},
    );

    builder(
      null,
      AsyncSnapshot.withData(
        ConnectionState.done,
        Resource.loading(),
      ),
    );

    expect(called, isTrue);
  });

  test(
      'calls loadingBuilder when resource is loading and promptLoading '
      'is null', () {
    var called = false;
    final builder = _TestClient().buildForStream(
      contentBuilder: null,
      errorBuilder: null,
      loadingBuilder: () {
        called = true;
      },
      emptyBuilder: null,
    );

    builder(
      null,
      AsyncSnapshot.withData(
        ConnectionState.done,
        Resource.loading('some data'),
      ),
    );

    expect(called, isTrue);
  });

  test('calls emptyBuilder if resource is empty', () {
    var called = false;
    final builder = _TestClient().buildForStream(
      contentBuilder: null,
      errorBuilder: null,
      loadingBuilder: null,
      emptyBuilder: () {
        called = true;
      },
    );

    builder(
      null,
      AsyncSnapshot.withData(
        ConnectionState.done,
        Resource.empty(),
      ),
    );

    expect(called, isTrue);
  });

  test(
      'calls promptError when resource has error, but has data, and '
      'promptError is not null', () {
    var called = false;
    final builder = _TestClient().buildForStream(
      contentBuilder: (data) {},
      errorBuilder: null,
      loadingBuilder: null,
      emptyBuilder: null,
      promptError: (error) {
        called = true;
      },
    );

    builder(
      null,
      AsyncSnapshot.withData(
        ConnectionState.done,
        Resource.error('error', 'some data'),
      ),
    );

    expect(called, isTrue);
  });

  test('calls errorBuilder when resource has error and has no data', () {
    var called = false;
    final builder = _TestClient().buildForStream(
      contentBuilder: null,
      errorBuilder: (error) {
        called = true;
      },
      loadingBuilder: null,
      emptyBuilder: null,
      promptError: (error) {},
    );

    builder(
      null,
      AsyncSnapshot.withData(
        ConnectionState.done,
        Resource.error('error'),
      ),
    );

    expect(called, isTrue);
  });

  test('calls errorBuilder when resource has error and promptError is null',
      () {
    var called = false;
    final builder = _TestClient().buildForStream(
      contentBuilder: null,
      errorBuilder: (error) {
        called = true;
      },
      loadingBuilder: null,
      emptyBuilder: null,
    );

    builder(
      null,
      AsyncSnapshot.withData(
        ConnectionState.done,
        Resource.error('error', 'some data'),
      ),
    );

    expect(called, isTrue);
  });

  test('calls contentBuilder if data is not null', () {
    var called = false;
    final builder = _TestClient().buildForStream(
      contentBuilder: (data) {
        called = true;
      },
      errorBuilder: null,
      loadingBuilder: null,
      emptyBuilder: null,
    );

    builder(
      null,
      AsyncSnapshot.withData(
        ConnectionState.done,
        Resource.success('some data'),
      ),
    );

    expect(called, isTrue);
  });
}
